"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var helper_plugin_utils_1 = require("@babel/helper-plugin-utils");
var template_1 = __importDefault(require("@babel/template"));
var t = __importStar(require("@babel/types"));
var utils_1 = require("@compiled/utils");
var is_automatic_runtime_1 = require("./utils/is-automatic-runtime");
var is_cc_component_1 = require("./utils/is-cc-component");
var is_create_element_1 = require("./utils/is-create-element");
var remove_style_declarations_1 = require("./utils/remove-style-declarations");
var to_uri_component_1 = require("./utils/to-uri-component");
exports.default = (0, helper_plugin_utils_1.declare)(function (api) {
    api.assertVersion(7);
    return {
        name: '@compiled/babel-plugin-strip-runtime',
        pre: function () {
            this.styleRules = [];
        },
        visitor: {
            Program: {
                exit: function (path, _a) {
                    var _this = this;
                    var _b;
                    var file = _a.file;
                    if (this.opts.compiledRequireExclude) {
                        // Rather than inserting styleRules to the code, inserting them to matadata in the case like SSR
                        if (!((_b = file.metadata) === null || _b === void 0 ? void 0 : _b.styleRules))
                            file.metadata.styleRules = [];
                        this.styleRules.forEach(function (rule) {
                            file.metadata.styleRules.push(rule);
                        });
                        return;
                    }
                    if (this.opts.styleSheetPath) {
                        (0, utils_1.preserveLeadingComments)(path);
                        this.styleRules.forEach(function (rule) {
                            // Each found atomic rule will create a new import that uses the styleSheetPath provided.
                            // The benefit is two fold:
                            // (1) thread safe collection of styles
                            // (2) caching -- resulting in faster builds (one import per rule!)
                            var params = (0, to_uri_component_1.toURIComponent)(rule);
                            path.unshiftContainer('body', template_1.default.ast("require(\"".concat(_this.opts.styleSheetPath, "?style=").concat(params, "\");")));
                            // We use require instead of import so it works with both ESM and CJS source.
                            // If we used ESM it would blow up with CJS source, unfortunately.
                        });
                    }
                },
            },
            ImportSpecifier: function (path) {
                if (t.isIdentifier(path.node.imported) && ['CC', 'CS'].includes(path.node.imported.name)) {
                    path.remove();
                }
            },
            JSXElement: function (path, pass) {
                if (!t.isJSXIdentifier(path.node.openingElement.name)) {
                    return;
                }
                var componentName = path.node.openingElement.name.name;
                if (componentName !== 'CC') {
                    return;
                }
                var _a = path.get('children'), compiledStyles = _a[1], nodeToReplace = _a[3];
                // Before we replace this node with its children we need to go through and remove all the
                // style declarations from the CS call.
                (0, remove_style_declarations_1.removeStyleDeclarations)(compiledStyles.node, path, pass);
                if (t.isJSXExpressionContainer(nodeToReplace.node)) {
                    var container = nodeToReplace;
                    path.replaceWith(container.node.expression);
                }
                else {
                    path.replaceWith(nodeToReplace);
                }
                // All done! Let's replace this node with the user land child.
                path.node.leadingComments = null;
                return;
            },
            CallExpression: function (path, pass) {
                var callee = path.node.callee;
                if ((0, is_create_element_1.isCreateElement)(callee)) {
                    // We've found something that looks like React.createElement(...)
                    // Now we want to check if it's from the Compiled Runtime and if it is - replace with its children.
                    var component = path.node.arguments[0];
                    if (!(0, is_cc_component_1.isCCComponent)(component)) {
                        return;
                    }
                    var _a = path.get('arguments'), compiledStyles = _a[2], nodeToReplace = _a[3];
                    // Before we replace this node with its children we need to go through and remove all the
                    // style declarations from the CS call.
                    (0, remove_style_declarations_1.removeStyleDeclarations)(compiledStyles.node, path, pass);
                    // All done! Let's replace this node with the user land child.
                    path.replaceWith(nodeToReplace);
                    path.node.leadingComments = null;
                    return;
                }
                if ((0, is_automatic_runtime_1.isAutomaticRuntime)(path.node, 'jsxs')) {
                    // We've found something that looks like _jsxs(...)
                    // Now we want to check if it's from the Compiled Runtime and if it is - replace with its children.
                    var component = path.node.arguments[0];
                    if (!(0, is_cc_component_1.isCCComponent)(component)) {
                        return;
                    }
                    var _b = path.get('arguments'), props = _b[1];
                    if (!t.isObjectExpression(props.node)) {
                        return;
                    }
                    var children = props.node.properties.find(function (prop) {
                        return (t.isObjectProperty(prop) && t.isIdentifier(prop.key) && prop.key.name === 'children');
                    });
                    if (!children || !t.isArrayExpression(children.value)) {
                        return;
                    }
                    var _c = children.value.elements, compiledStyles = _c[0], nodeToReplace = _c[1];
                    if (!t.isExpression(nodeToReplace) || !t.isExpression(compiledStyles)) {
                        throw new Error('Nodes should be expressions.');
                    }
                    // Before we replace this node with its children we need to go through and remove all the
                    // style declarations from the CS call.
                    (0, remove_style_declarations_1.removeStyleDeclarations)(compiledStyles, path, pass);
                    // All done! Let's replace this node with the user land child.
                    path.replaceWith(nodeToReplace);
                    path.node.leadingComments = null;
                    return;
                }
            },
        },
    };
});
//# sourceMappingURL=index.js.map